@@ -24,6 +24,7 @@
 #include <efi.h>
 
 #include <kernel_efivars.h>
+#include <openssl_sign.h>
 #include <guid.h>
 #include <sha256.h>
 #include <version.h>
@@ -51,6 +52,7 @@ help(const char *progname)
 	       "\t-g <guid>\tOptional <guid> for the X509 Certificate\n"
 	       "\t-k <key>\tSecret key file for authorising User Mode updates\n"
 	       "\t-d <list>[-<entry>]\tDelete the signature list <list> (or just a single <entry> within the list)\n"
+	       "\t--engine <eng>\tUse engine <eng> for private key\n"
 	       );
 }
 
@@ -59,6 +61,7 @@ main(int argc, char *argv[])
 {
 	char *variables[] = { "PK", "KEK", "db", "dbx" };
 	char *signedby[] = { "PK", "PK", "KEK", "KEK" };
+	char *engine = NULL;
 	EFI_GUID *owners[] = { &GV_GUID, &GV_GUID, &SIG_DB, &SIG_DB };
 	EFI_GUID *owner, guid = MOK_OWNER;
 	int i, esl_mode = 0, fd, ret, delsig = -1, delentry = -1;
@@ -113,6 +116,10 @@ main(int argc, char *argv[])
 			sscanf(argv[2], "%d-%d", &delsig, &delentry);
 			argv += 2;
 			argc -= 2;
+		} else if (strcmp(argv[1], "--engine") == 0) {
+			engine = argv[2];
+			argv += 2;
+			argc -= 2;
 		} else {
 			/* unrecognised option */
 			break;
@@ -279,8 +286,7 @@ main(int argc, char *argv[])
 			fprintf(stderr, "Can't update variable%s without a key\n", variable_is_setupmode() ? "" : " in User Mode");
 			exit(1);
 		}
-		BIO *key = BIO_new_file(key_file, "r");
-		EVP_PKEY *pkey = PEM_read_bio_PrivateKey(key, NULL, NULL, NULL);
+		EVP_PKEY *pkey = read_private_key(engine, key_file);
 		if (!pkey) {
 			fprintf(stderr, "error reading private key %s\n", key_file);
 			exit(1);
@@ -321,6 +327,9 @@ main(int argc, char *argv[])
 
 		EFI_TIME timestamp;
 		time_t t;
+		unsigned char *tmp;
+		int sigsize;
+
 		struct tm *tm;
 		memset(&timestamp, 0, sizeof(timestamp));
 		time(&t);
@@ -353,17 +362,7 @@ main(int argc, char *argv[])
 		ptr += sizeof(timestamp);
 		memcpy(ptr, buf, st.st_size);
 
-		BIO *bio = BIO_new_mem_buf(signbuf, signbuflen);
-		PKCS7 *p7 = PKCS7_sign(NULL, NULL, NULL, bio,
-				       PKCS7_BINARY | PKCS7_PARTIAL
-				       | PKCS7_DETACHED);
-		const EVP_MD *md = EVP_get_digestbyname("SHA256");
-		PKCS7_sign_add_signer(p7, X, pkey, md, PKCS7_BINARY
-				      | PKCS7_DETACHED);
-		PKCS7_final(p7, bio, PKCS7_BINARY | PKCS7_DETACHED);
-
-
-		int sigsize = i2d_PKCS7(p7, NULL);
+		sign_efi_var_ssl(signbuf, signbuflen, pkey, X, &tmp, &sigsize);
 
 		EFI_VARIABLE_AUTHENTICATION_2 *var_auth = malloc(sizeof(EFI_VARIABLE_AUTHENTICATION_2) + sigsize);
 		var_auth->TimeStamp = timestamp;
@@ -371,8 +370,7 @@ main(int argc, char *argv[])
 		var_auth->AuthInfo.Hdr.dwLength = sigsize + OFFSET_OF(WIN_CERTIFICATE_UEFI_GUID, CertData);
 		var_auth->AuthInfo.Hdr.wRevision = 0x0200;
 		var_auth->AuthInfo.Hdr.wCertificateType = WIN_CERT_TYPE_EFI_GUID;
-		unsigned char *tmp = var_auth->AuthInfo.CertData;
-		i2d_PKCS7(p7, &tmp);
+		memcpy(var_auth->AuthInfo.CertData, tmp, sigsize);
 		ERR_print_errors_fp(stderr);
 
 		/* new update now consists of two parts: the
